from fastapi import APIRouter, Depends, HTTPException, UploadFile, File, Form
from sqlmodel import Session
from backend.models.models import Shop, User
from backend.schemas.shop import ShopCreate, ShopRead
from backend.database import get_session
from backend.routes.auth import get_current_user
from core.config import settings
import shutil
import os

router = APIRouter()


@router.post("/create", response_model=ShopRead)
def create_shop(
    name: str = Form(...),
    description: str = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    shop = ShopCreate(name=name, description=description, owner_id=current_user.id)
    db_shop = Shop.from_orm(shop)

    session.add(db_shop)
    session.commit()
    session.refresh(db_shop)

    shop_dir = os.path.join(settings.static_dir, f"shop_{db_shop.name}")
    os.makedirs(shop_dir, exist_ok=True)

    if file and file.filename:
        file_location = os.path.join(shop_dir, file.filename)
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)
        db_shop.image_url = file_location
    else:
        db_shop.image_url = os.path.join(
            settings.static_dir, "default/default_shop.png"
        )

    return db_shop


@router.get("/list", response_model=list[ShopRead])
def get_all_shops(session: Session = Depends(get_session)):
    shops = session.query(Shop).all()
    return shops


@router.get("/get/{shop_id}", response_model=ShopRead)
def read_shop(shop_id: int, session: Session = Depends(get_session)):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    return shop


@router.put("/put/{shop_id}", response_model=ShopRead)
def update_shop(
    shop_id: int,
    name: str = Form(None),
    description: str = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    db_shop = session.get(Shop, shop_id)
    if not db_shop:
        raise HTTPException(status_code=404, detail="Shop not found")

    # Ensure the current user is the shop owner
    if db_shop.owner_id != current_user.id:
        raise HTTPException(status_code=403, detail="Unauthorized to update this shop")

    if name:
        db_shop.name = name
    if description:
        db_shop.description = description

    shop_dir = os.path.join(settings.static_dir, f"shop_{db_shop.name}")
    os.makedirs(shop_dir, exist_ok=True)

    if file and file.filename:
        file_location = os.path.join(shop_dir, file.filename)
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)
        db_shop.image_url = file_location
    else:
        db_shop.image_url = os.path.join(
            settings.static_dir, "default/default_shop.png"
        )

    session.add(db_shop)
    session.commit()
    session.refresh(db_shop)

    return db_shop


@router.delete("/delete/{shop_id}")
def delete_shop(
    shop_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")

    # Ensure the current user is the shop owner
    if shop.owner_id != current_user.id:
        raise HTTPException(status_code=403, detail="Unauthorized to delete this shop")

    session.delete(shop)
    session.commit()
    return {"message": "Shop deleted successfully"}
